/* (C) 2022 YiRing, Inc. */
package com.yiring.app.param.dept;

import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.user.User;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import javax.validation.constraints.NotNull;
import lombok.*;
import lombok.experimental.FieldDefaults;

/**
 * 部门新增入参
 * @author LJ-2204
 * @date 2022/4/19
 */

@ApiModel("DepartmentAddParam")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldDefaults(level = AccessLevel.PRIVATE)
public class DepartmentAddParam implements Serializable {

    @Serial
    private static final long serialVersionUID = -9143398676976586072L;

    @ApiModelProperty(value = "上级部门", example = "1", required = true)
    @NotNull
    Long pid;

    @ApiModelProperty(value = "名称", example = "Java", required = true)
    @NotNull
    String name;

    @ApiModelProperty(value = "负责人", example = "1")
    Long leaderId;

    @ApiModelProperty(value = "部门状态", example = "1")
    Boolean enable;

    public Department transform() {
        return Department
            .builder()
            .pid(this.pid)
            .name(this.name)
            .leader(User.builder().id(this.leaderId).build())
            .enable(this.enable)
            .createTime(LocalDateTime.now())
            .updateTime(LocalDateTime.now())
            .build();
    }
}
