/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.domain.dept;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.yiring.auth.domain.post.Post;
import com.yiring.auth.domain.user.User;
import com.yiring.common.annotation.FieldMapping;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.Hibernate;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.snowflake.SnowflakeId;

/**
 * 部门
 *
 * @author ifzm
 * 2018/9/3 17:08
 */

@Getter
@Setter
@ToString
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Comment("系统部门")
@Table(name = "SYS_DEPARTMENT", indexes = { @Index(columnList = "pid"), @Index(columnList = "tree") })
public class Department implements Serializable {

    @Serial
    private static final long serialVersionUID = -5402731145404250150L;

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.LONG)
    Long id;

    /**
     * 数据来源于【真源人员定位系统 - 部门】
     * 作用: 用于双向联动进行数据同步
     */
    @FieldMapping("id")
    @Comment("外链主键")
    Long linkId;

    @Comment("名称")
    String name;

    @Comment("描述")
    String describe;

    @ManyToOne
    @JoinColumn(name = "leader_id")
    @JsonIgnore
    @Comment("负责人")
    User leader;

    @Comment("序号")
    Integer serial;

    @Comment("层级")
    Integer level;

    @Comment("是否启用")
    Boolean enable;

    @Comment("是否删除")
    Boolean deleted;

    @Comment("父级ID")
    Long pid;

    @Comment("树节点标识")
    String tree;

    @Comment("更新时间")
    LocalDateTime updateTime;

    @Comment("创建时间")
    LocalDateTime createTime;

    @Comment("岗位集合")
    @Builder.Default
    @ToString.Exclude
    @ManyToMany
    @JoinTable(
        name = "SYS_DEPARTMENT_POSTS",
        joinColumns = { @JoinColumn(name = "department_id") },
        inverseJoinColumns = { @JoinColumn(name = "post_id") }
    )
    Set<Post> posts = new HashSet<>(0);

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || Hibernate.getClass(this) != Hibernate.getClass(o)) return false;
        Department that = (Department) o;
        return id != null && Objects.equals(id, that.id);
    }

    @Override
    public int hashCode() {
        return getClass().hashCode();
    }
}
